import sys
import traceback
from typing import List

import urllib.parse
from django.core.management.base import BaseCommand

from ... import models, remote

MAX_ERRORS = 5


class Command(BaseCommand):
    help = 'Sync data from external databases'

    def add_arguments(self, parser):
        return None

    def handle(self, *args, **options):
        failed_urls: List[str] = list()
        for convention_ in models.ConventionSeries.objects.all():
            convention: models.ConventionSeries = convention_
            del convention_
            if len(convention.clone_paused_until_gets_200_url or '') > 0 and convention.clone_paused_until_gets_200:
                if not remote.http_get_returns_200(convention.clone_paused_until_gets_200_url):
                    failed_urls.append(
                        convention.clone_paused_until_gets_200_url)
                    continue
                else:
                    convention.clone_paused_until_gets_200 = False
                    convention.save()
            if (handler := remote.schedule_sync_handlers.get(convention.clone_type)) is not None:
                clone_idtf = f'{convention.clone_type}://{convention.clone_from}'
                try:
                    handler(convention.clone_from, convention.featured)
                    models.SyncFailure.delete_if_exists(clone_idtf)
                except Exception:
                    error_count = models.SyncFailure.get_or_create_and_increment(
                        clone_idtf).count
                    emsg = f'Schedule sync error @ {convention.name.en}: {error_count}/{MAX_ERRORS}'
                    print(emsg)
                    if error_count > MAX_ERRORS:
                        print(emsg, file=sys.stderr)
                        traceback.print_exc()
            if hasattr(convention, 'telegram_channel') and convention.telegram_channel is not None and convention.telegram_channel != "":
                tg_idtf = f'https://t.me/s/{convention.telegram_channel}'
                try:
                    remote.announcement_sync_handlers['telegram_channel'](
                        convention)
                    models.SyncFailure.delete_if_exists(tg_idtf)
                except Exception:
                    error_count = models.SyncFailure.get_or_create_and_increment(
                        tg_idtf).count
                    emsg = f'Announcements sync error @ Telegram Channel @{convention.name.en}: {error_count}/{MAX_ERRORS}'
                    print(emsg)
                    if error_count > MAX_ERRORS:
                        print(emsg, file=sys.stderr)
                        traceback.print_exc()
        for sync_url in [
                o
                for x in [
                    models.SyncUrlForLanguagePhrase,
                    models.SyncUrlForLanguageText,
                    models.SyncUrlForLanguageRichText,
                ]
                for o in x.objects.all()
        ]:
            to_skip = False
            for failed_url_ in failed_urls:
                for failed_url in [failed_url_, urllib.parse.quote(failed_url_), urllib.parse.quote_plus(failed_url_)]:
                    if (
                        (failed_url in sync_url.clone_paused_until_gets_200_url) or
                        (failed_url in sync_url.url)
                    ):
                        to_skip = True
                        break
                if to_skip:
                    break
            if not to_skip and len(sync_url.clone_paused_until_gets_200_url or '') > 0 and sync_url.clone_paused_until_gets_200:
                if not remote.http_get_returns_200(sync_url.clone_paused_until_gets_200_url):
                    failed_urls.append(
                        sync_url.clone_paused_until_gets_200_url)
                    to_skip = True
                else:
                    sync_url.clone_paused_until_gets_200 = False
                    sync_url.save()
            if to_skip:
                print(
                    f'Skipped language string @ {sync_url.associated_language.reminder_alias[:50]}')
            else:
                url_idtf = sync_url.url
                try:
                    remote.update_language_from(
                        sync_url.associated_language, sync_url.url)
                    print(
                        f'Synced language string @ {sync_url.associated_language.reminder_alias[:50]}')
                    models.SyncFailure.delete_if_exists(url_idtf)
                except Exception:
                    error_count = models.SyncFailure.get_or_create_and_increment(
                        url_idtf).count
                    emsg = f'Language string sync error @ {sync_url.associated_language.reminder_alias[:50]}: {error_count}/{MAX_ERRORS}'
                    print(emsg)
                    if error_count > MAX_ERRORS:
                        print(emsg, file=sys.stderr)
                        traceback.print_exc()
