import sys
import traceback
from typing import List

import urllib.parse
from django.core.management.base import BaseCommand
from django.conf import settings
import urllib.parse
import requests
import hashlib

from ... import models, remote

MAX_ERRORS = 5
API_URL = "https://word2vec.furmeet.app/convert.json?key={}&lang={}&sentence={}"


def generate_api_link(lang, text):
    return API_URL.format(settings.SHARED_KEY, lang, urllib.parse.quote_plus(text))


def get_api_vec(lang, text):
    resp = requests.get(generate_api_link(lang, text))
    resp.raise_for_status()
    return resp.json()


class Command(BaseCommand):
    help = 'Translate word2vec'

    def add_arguments(self, parser):
        return None

    def handle(self, *args, **options):
        for activity in models.ConventionActivity.objects.all():
            lang = activity.convention.language
            t = getattr(activity.title, lang, '')
            s = getattr(activity.subtitle, lang, '')
            d = getattr(activity.description, lang, '')
            k = f'{t}\n{s}\n{d}'
            m = hashlib.sha3_512()
            m.update(k.encode(encoding='utf-8'))
            h = m.digest()
            w = bytes(activity.word2vec_original_data_hash)
            if h != w:
                activity.word2vec_original_data_hash = h
                activity.word2vec = get_api_vec(lang, k)
                activity.save()
