# Create your views here.
import json
import gzip
import datetime
from pathlib import Path
from django.conf import settings
from django.views import View
from django.http import FileResponse
from django.http import HttpResponse
from django.http import JsonResponse
from django.shortcuts import get_object_or_404
from django.views.decorators.http import last_modified

from . import models
from .modelconversors import DjangoModelConversorToNativeStructures as DMCTNS


class PingPongView(View):
    def get(self, request):
        with Path('.').joinpath(settings.STATIC_URL.strip('/')).joinpath('ping.txt').open() as f:
            return HttpResponse(f.read(), content_type='text/plain')


class ConventionsJsonView(View):
    def get(self, request, *args, **kwargs):
        jc = Path(models.settings.STATIC_ROOT).joinpath('json_cache')
        fl = jc.joinpath(f'convention_{kwargs["url"]}.json')
        flz = jc.joinpath(f'convention_{kwargs["url"]}.json.gz')
        if not jc.exists():
            jc.mkdir(parents=True, exist_ok=True)
        if not fl.exists():
            root = get_object_or_404(
                models.ConventionSeries, url=kwargs.get('url'))
            data = DMCTNS.convention_series(root)
            json_pretty_print_settings = dict()
            srz = json.dumps(data, **json_pretty_print_settings)
            fl.write_text(srz)
            with flz.open('wb') as f:
                with gzip.GzipFile(mode='wb', compresslevel=9, fileobj=f) as g:
                    g.write(srz.encode())

        def from_disk(rq):
            if 'gzip' in rq.META.get('HTTP_ACCEPT_ENCODING', ''):
                r = FileResponse(
                    flz.open('rb'), content_type='application/json')
                r['Content-Encoding'] = 'gzip'
                return r
            else:
                return FileResponse(fl.open('rb'), content_type='application/json')
        wrapped = last_modified(
            lambda rq: datetime.datetime.fromtimestamp(fl.stat().st_mtime))(from_disk)
        return wrapped(request)


class ConventionsListApiView(View):
    def get(self, request):
        return JsonResponse(sorted([x.url for x in models.ConventionSeries.objects.all()]))
